@@ -17,9 +17,7 @@
  * under the License.
  */
 
-#include <arrow-glib/error.hpp>
-#include <arrow-glib/ipc-options.hpp>
-#include <arrow-glib/schema.hpp>
+#include <arrow-glib/arrow-glib.hpp>
 
 #include <arrow-flight-glib/common.hpp>
 
@@ -36,7 +34,7 @@ G_BEGIN_DECLS
  * #GAFlightLocation is a class for location.
  *
  * #GAFlightDescriptor is a base class for all descriptor classes such
- * as #GArrowFlightPathDescriptor.
+ * as #GAFlightPathDescriptor.
  *
  * #GAFlightPathDescriptor is a class for path descriptor.
  *
@@ -48,10 +46,33 @@ G_BEGIN_DECLS
  *
  * #GAFlightInfo is a class for flight information.
  *
+ * #GAFlightStreamChunk is a class for a chunk in stream.
+ *
+ * #GAFlightRecordBatchReader is an abstract class for reading record
+ * batches with metadata.
+ *
+ * #GAFlightRecordBatchWeriter is an abstract class for
+ * writing record batches with metadata.
+ *
  * Since: 5.0.0
  */
 
-typedef struct GAFlightCriteriaPrivate_ {
+/**
+ * GAFlightHeaderFunc:
+ * @name: A header name.
+ * @value: The value corresponding to the name.
+ * @user_data: User data passed to gaflight_call_options_foreach_header()
+ *   and so on.
+ *
+ * It is called with each header name/value pair, together with the
+ * @user_data parameter which is passed to
+ * gaflight_call_options_foreach_header() and so on.
+ *
+ * Since: 9.0.0
+ */
+
+typedef struct GAFlightCriteriaPrivate_
+{
   arrow::flight::Criteria criteria;
   GBytes *expression;
 } GAFlightCriteriaPrivate;
@@ -60,14 +81,11 @@ enum {
   PROP_EXPRESSION = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightCriteria,
-                           gaflight_criteria,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightCriteria, gaflight_criteria, G_TYPE_OBJECT)
 
-#define GAFLIGHT_CRITERIA_GET_PRIVATE(obj)            \
-  static_cast<GAFlightCriteriaPrivate *>(             \
-    gaflight_criteria_get_instance_private(           \
-      GAFLIGHT_CRITERIA(obj)))
+#define GAFLIGHT_CRITERIA_GET_PRIVATE(obj)                                               \
+  static_cast<GAFlightCriteriaPrivate *>(                                                \
+    gaflight_criteria_get_instance_private(GAFLIGHT_CRITERIA(obj)))
 
 static void
 gaflight_criteria_dispose(GObject *object)
@@ -109,8 +127,7 @@ gaflight_criteria_set_property(GObject *object,
     {
       gsize size;
       auto data = g_bytes_get_data(priv->expression, &size);
-      priv->criteria.expression.assign(static_cast<const char *>(data),
-                                       size);
+      priv->criteria.expression.assign(static_cast<const char *>(data), size);
     }
     break;
   default:
@@ -141,7 +158,7 @@ static void
 gaflight_criteria_init(GAFlightCriteria *object)
 {
   auto priv = GAFLIGHT_CRITERIA_GET_PRIVATE(object);
-  new(&priv->criteria) arrow::flight::Criteria;
+  new (&priv->criteria) arrow::flight::Criteria;
 }
 
 static void
@@ -183,24 +200,19 @@ GAFlightCriteria *
 gaflight_criteria_new(GBytes *expression)
 {
   return GAFLIGHT_CRITERIA(
-    g_object_new(GAFLIGHT_TYPE_CRITERIA,
-                 "expression", expression,
-                 NULL));
+    g_object_new(GAFLIGHT_TYPE_CRITERIA, "expression", expression, NULL));
 }
 
-
-typedef struct GAFlightLocationPrivate_ {
+typedef struct GAFlightLocationPrivate_
+{
   arrow::flight::Location location;
 } GAFlightLocationPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightLocation,
-                           gaflight_location,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightLocation, gaflight_location, G_TYPE_OBJECT)
 
-#define GAFLIGHT_LOCATION_GET_PRIVATE(obj)            \
-  static_cast<GAFlightLocationPrivate *>(             \
-    gaflight_location_get_instance_private(           \
-      GAFLIGHT_LOCATION(obj)))
+#define GAFLIGHT_LOCATION_GET_PRIVATE(obj)                                               \
+  static_cast<GAFlightLocationPrivate *>(                                                \
+    gaflight_location_get_instance_private(GAFLIGHT_LOCATION(obj)))
 
 static void
 gaflight_location_finalize(GObject *object)
@@ -216,7 +228,7 @@ static void
 gaflight_location_init(GAFlightLocation *object)
 {
   auto priv = GAFLIGHT_LOCATION_GET_PRIVATE(object);
-  new(&priv->location) arrow::flight::Location;
+  new (&priv->location) arrow::flight::Location;
 }
 
 static void
@@ -237,13 +249,12 @@ gaflight_location_class_init(GAFlightLocationClass *klass)
  * Since: 5.0.0
  */
 GAFlightLocation *
-gaflight_location_new(const gchar *uri,
-                      GError **error)
+gaflight_location_new(const gchar *uri, GError **error)
 {
   auto location = GAFLIGHT_LOCATION(g_object_new(GAFLIGHT_TYPE_LOCATION, NULL));
   auto flight_location = gaflight_location_get_raw(location);
   if (garrow::check(error,
-                    arrow::flight::Location::Parse(uri, flight_location),
+                    arrow::flight::Location::Parse(uri).Value(flight_location),
                     "[flight-location][new]")) {
     return location;
   } else {
@@ -266,7 +277,8 @@ gchar *
 gaflight_location_to_string(GAFlightLocation *location)
 {
   const auto flight_location = gaflight_location_get_raw(location);
-  return g_strdup(flight_location->ToString().c_str());
+  const auto string = flight_location->ToString();
+  return g_strdup(string.c_str());
 }
 
 /**
@@ -283,7 +295,8 @@ gchar *
 gaflight_location_get_scheme(GAFlightLocation *location)
 {
   const auto flight_location = gaflight_location_get_raw(location);
-  return g_strdup(flight_location->scheme().c_str());
+  const auto scheme = flight_location->scheme();
+  return g_strdup(scheme.c_str());
 }
 
 /**
@@ -296,16 +309,15 @@ gaflight_location_get_scheme(GAFlightLocation *location)
  * Since: 5.0.0
  */
 gboolean
-gaflight_location_equal(GAFlightLocation *location,
-                        GAFlightLocation *other_location)
+gaflight_location_equal(GAFlightLocation *location, GAFlightLocation *other_location)
 {
   const auto flight_location = gaflight_location_get_raw(location);
   const auto flight_other_location = gaflight_location_get_raw(other_location);
   return flight_location->Equals(*flight_other_location);
 }
 
-
-typedef struct GAFlightDescriptorPrivate_ {
+typedef struct GAFlightDescriptorPrivate_
+{
   arrow::flight::FlightDescriptor descriptor;
 } GAFlightDescriptorPrivate;
 
@@ -317,10 +329,9 @@ G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE(GAFlightDescriptor,
                                     gaflight_descriptor,
                                     G_TYPE_OBJECT)
 
-#define GAFLIGHT_DESCRIPTOR_GET_PRIVATE(obj)            \
-  static_cast<GAFlightDescriptorPrivate *>(             \
-    gaflight_descriptor_get_instance_private(           \
-      GAFLIGHT_DESCRIPTOR(obj)))
+#define GAFLIGHT_DESCRIPTOR_GET_PRIVATE(obj)                                             \
+  static_cast<GAFlightDescriptorPrivate *>(                                              \
+    gaflight_descriptor_get_instance_private(GAFLIGHT_DESCRIPTOR(obj)))
 
 static void
 gaflight_descriptor_finalize(GObject *object)
@@ -342,8 +353,8 @@ gaflight_descriptor_set_property(GObject *object,
 
   switch (prop_id) {
   case PROP_DESCRIPTOR:
-    priv->descriptor = *static_cast<arrow::flight::FlightDescriptor *>(
-      g_value_get_pointer(value));
+    priv->descriptor =
+      *static_cast<arrow::flight::FlightDescriptor *>(g_value_get_pointer(value));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -355,7 +366,7 @@ static void
 gaflight_descriptor_init(GAFlightDescriptor *object)
 {
   auto priv = GAFLIGHT_DESCRIPTOR_GET_PRIVATE(object);
-  new(&priv->descriptor) arrow::flight::FlightDescriptor;
+  new (&priv->descriptor) arrow::flight::FlightDescriptor;
 }
 
 static void
@@ -367,11 +378,11 @@ gaflight_descriptor_class_init(GAFlightDescriptorClass *klass)
   gobject_class->set_property = gaflight_descriptor_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("descriptor",
-                              "Descriptor",
-                              "The raw arrow::flight::FlightDescriptor",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "descriptor",
+    "Descriptor",
+    "The raw arrow::flight::FlightDescriptor",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_EXPRESSION, spec);
 }
 
@@ -388,8 +399,9 @@ gaflight_descriptor_class_init(GAFlightDescriptorClass *klass)
 gchar *
 gaflight_descriptor_to_string(GAFlightDescriptor *descriptor)
 {
-  auto flight_descriptor = gaflight_descriptor_get_raw(descriptor);
-  return g_strdup(flight_descriptor->ToString().c_str());
+  const auto flight_descriptor = gaflight_descriptor_get_raw(descriptor);
+  const auto string = flight_descriptor->ToString();
+  return g_strdup(string.c_str());
 }
 
 /**
@@ -406,17 +418,12 @@ gboolean
 gaflight_descriptor_equal(GAFlightDescriptor *descriptor,
                           GAFlightDescriptor *other_descriptor)
 {
-  const auto flight_descriptor =
-    gaflight_descriptor_get_raw(descriptor);
-  const auto flight_other_descriptor =
-    gaflight_descriptor_get_raw(other_descriptor);
+  const auto flight_descriptor = gaflight_descriptor_get_raw(descriptor);
+  const auto flight_other_descriptor = gaflight_descriptor_get_raw(other_descriptor);
   return flight_descriptor->Equals(*flight_other_descriptor);
 }
 
-
-G_DEFINE_TYPE(GAFlightPathDescriptor,
-              gaflight_path_descriptor,
-              GAFLIGHT_TYPE_DESCRIPTOR)
+G_DEFINE_TYPE(GAFlightPathDescriptor, gaflight_path_descriptor, GAFLIGHT_TYPE_DESCRIPTOR)
 
 static void
 gaflight_path_descriptor_init(GAFlightPathDescriptor *object)
@@ -439,16 +446,14 @@ gaflight_path_descriptor_class_init(GAFlightPathDescriptorClass *klass)
  * Since: 5.0.0
  */
 GAFlightPathDescriptor *
-gaflight_path_descriptor_new(const gchar **paths,
-                             gsize n_paths)
+gaflight_path_descriptor_new(const gchar **paths, gsize n_paths)
 {
   std::vector<std::string> flight_paths;
   for (gsize i = 0; i < n_paths; i++) {
     flight_paths.push_back(paths[i]);
   }
   auto flight_descriptor = arrow::flight::FlightDescriptor::Path(flight_paths);
-  return GAFLIGHT_PATH_DESCRIPTOR(
-    gaflight_descriptor_new_raw(&flight_descriptor));
+  return GAFLIGHT_PATH_DESCRIPTOR(gaflight_descriptor_new_raw(&flight_descriptor));
 }
 
 /**
@@ -481,7 +486,6 @@ gaflight_path_descriptor_get_paths(GAFlightPathDescriptor *descriptor)
   }
 }
 
-
 G_DEFINE_TYPE(GAFlightCommandDescriptor,
               gaflight_command_descriptor,
               GAFLIGHT_TYPE_DESCRIPTOR)
@@ -508,8 +512,7 @@ GAFlightCommandDescriptor *
 gaflight_command_descriptor_new(const gchar *command)
 {
   auto flight_descriptor = arrow::flight::FlightDescriptor::Command(command);
-  return GAFLIGHT_COMMAND_DESCRIPTOR(
-    gaflight_descriptor_new_raw(&flight_descriptor));
+  return GAFLIGHT_COMMAND_DESCRIPTOR(gaflight_descriptor_new_raw(&flight_descriptor));
 }
 
 /**
@@ -531,8 +534,8 @@ gaflight_command_descriptor_get_command(GAFlightCommandDescriptor *descriptor)
   return g_strdup(flight_command.c_str());
 }
 
-
-typedef struct GAFlightTicketPrivate_ {
+typedef struct GAFlightTicketPrivate_
+{
   arrow::flight::Ticket ticket;
   GBytes *data;
 } GAFlightTicketPrivate;
@@ -541,14 +544,11 @@ enum {
   PROP_DATA = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightTicket,
-                           gaflight_ticket,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightTicket, gaflight_ticket, G_TYPE_OBJECT)
 
-#define GAFLIGHT_TICKET_GET_PRIVATE(obj)            \
-  static_cast<GAFlightTicketPrivate *>(             \
-    gaflight_ticket_get_instance_private(           \
-      GAFLIGHT_TICKET(obj)))
+#define GAFLIGHT_TICKET_GET_PRIVATE(obj)                                                 \
+  static_cast<GAFlightTicketPrivate *>(                                                  \
+    gaflight_ticket_get_instance_private(GAFLIGHT_TICKET(obj)))
 
 static void
 gaflight_ticket_dispose(GObject *object)
@@ -590,8 +590,7 @@ gaflight_ticket_set_property(GObject *object,
     {
       gsize size;
       auto data = g_bytes_get_data(priv->data, &size);
-      priv->ticket.ticket.assign(static_cast<const char *>(data),
-                                 size);
+      priv->ticket.ticket.assign(static_cast<const char *>(data), size);
     }
     break;
   default:
@@ -622,7 +621,7 @@ static void
 gaflight_ticket_init(GAFlightTicket *object)
 {
   auto priv = GAFLIGHT_TICKET_GET_PRIVATE(object);
-  new(&priv->ticket) arrow::flight::Ticket;
+  new (&priv->ticket) arrow::flight::Ticket;
 }
 
 static void
@@ -664,10 +663,7 @@ gaflight_ticket_class_init(GAFlightTicketClass *klass)
 GAFlightTicket *
 gaflight_ticket_new(GBytes *data)
 {
-  return GAFLIGHT_TICKET(
-    g_object_new(GAFLIGHT_TYPE_TICKET,
-                 "data", data,
-                 NULL));
+  return GAFLIGHT_TICKET(g_object_new(GAFLIGHT_TYPE_TICKET, "data", data, NULL));
 }
 
 /**
@@ -680,16 +676,15 @@ gaflight_ticket_new(GBytes *data)
  * Since: 5.0.0
  */
 gboolean
-gaflight_ticket_equal(GAFlightTicket *ticket,
-                      GAFlightTicket *other_ticket)
+gaflight_ticket_equal(GAFlightTicket *ticket, GAFlightTicket *other_ticket)
 {
   const auto flight_ticket = gaflight_ticket_get_raw(ticket);
   const auto flight_other_ticket = gaflight_ticket_get_raw(other_ticket);
   return flight_ticket->Equals(*flight_other_ticket);
 }
 
-
-typedef struct GAFlightEndpointPrivate_ {
+typedef struct GAFlightEndpointPrivate_
+{
   arrow::flight::FlightEndpoint endpoint;
   GAFlightTicket *ticket;
   GList *locations;
@@ -699,14 +694,11 @@ enum {
   PROP_TICKET = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightEndpoint,
-                           gaflight_endpoint,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightEndpoint, gaflight_endpoint, G_TYPE_OBJECT)
 
-#define GAFLIGHT_ENDPOINT_GET_PRIVATE(obj)            \
-  static_cast<GAFlightEndpointPrivate *>(             \
-    gaflight_endpoint_get_instance_private(           \
-      GAFLIGHT_ENDPOINT(obj)))
+#define GAFLIGHT_ENDPOINT_GET_PRIVATE(obj)                                               \
+  static_cast<GAFlightEndpointPrivate *>(                                                \
+    gaflight_endpoint_get_instance_private(GAFLIGHT_ENDPOINT(obj)))
 
 static void
 gaflight_endpoint_dispose(GObject *object)
@@ -758,7 +750,7 @@ static void
 gaflight_endpoint_init(GAFlightEndpoint *object)
 {
   auto priv = GAFLIGHT_ENDPOINT_GET_PRIVATE(object);
-  new(&priv->endpoint) arrow::flight::FlightEndpoint;
+  new (&priv->endpoint) arrow::flight::FlightEndpoint;
 }
 
 static void
@@ -796,8 +788,7 @@ gaflight_endpoint_class_init(GAFlightEndpointClass *klass)
  * Since: 5.0.0
  */
 GAFlightEndpoint *
-gaflight_endpoint_new(GAFlightTicket *ticket,
-                      GList *locations)
+gaflight_endpoint_new(GAFlightTicket *ticket, GList *locations)
 {
   auto endpoint = gaflight_endpoint_new_raw(nullptr, ticket);
   auto priv = GAFLIGHT_ENDPOINT_GET_PRIVATE(endpoint);
@@ -819,8 +810,7 @@ gaflight_endpoint_new(GAFlightTicket *ticket,
  * Since: 5.0.0
  */
 gboolean
-gaflight_endpoint_equal(GAFlightEndpoint *endpoint,
-                        GAFlightEndpoint *other_endpoint)
+gaflight_endpoint_equal(GAFlightEndpoint *endpoint, GAFlightEndpoint *other_endpoint)
 {
   const auto flight_endpoint = gaflight_endpoint_get_raw(endpoint);
   const auto flight_other_endpoint = gaflight_endpoint_get_raw(other_endpoint);
@@ -846,15 +836,14 @@ gaflight_endpoint_get_locations(GAFlightEndpoint *endpoint)
   const auto flight_endpoint = gaflight_endpoint_get_raw(endpoint);
   GList *locations = NULL;
   for (const auto &flight_location : flight_endpoint->locations) {
-    auto location = gaflight_location_new(flight_location.ToString().c_str(),
-                                          nullptr);
+    auto location = gaflight_location_new(flight_location.ToString().c_str(), nullptr);
     locations = g_list_prepend(locations, location);
   }
   return g_list_reverse(locations);
 }
 
-
-typedef struct GAFlightInfoPrivate_ {
+typedef struct GAFlightInfoPrivate_
+{
   arrow::flight::FlightInfo info;
 } GAFlightInfoPrivate;
 
@@ -862,14 +851,11 @@ enum {
   PROP_INFO = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightInfo,
-                           gaflight_info,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightInfo, gaflight_info, G_TYPE_OBJECT)
 
-#define GAFLIGHT_INFO_GET_PRIVATE(obj)            \
-  static_cast<GAFlightInfoPrivate *>(             \
-    gaflight_info_get_instance_private(           \
-      GAFLIGHT_INFO(obj)))
+#define GAFLIGHT_INFO_GET_PRIVATE(obj)                                                   \
+  static_cast<GAFlightInfoPrivate *>(                                                    \
+    gaflight_info_get_instance_private(GAFLIGHT_INFO(obj)))
 
 static void
 gaflight_info_finalize(GObject *object)
@@ -892,9 +878,8 @@ gaflight_info_set_property(GObject *object,
   switch (prop_id) {
   case PROP_INFO:
     {
-      auto info =
-        static_cast<arrow::flight::FlightInfo *>(g_value_get_pointer(value));
-      new(&(priv->info)) arrow::flight::FlightInfo(*info);
+      auto info = static_cast<arrow::flight::FlightInfo *>(g_value_get_pointer(value));
+      new (&(priv->info)) arrow::flight::FlightInfo(*info);
     }
     break;
   default:
@@ -917,11 +902,11 @@ gaflight_info_class_init(GAFlightInfoClass *klass)
   gobject_class->set_property = gaflight_info_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("info",
-                              "Info",
-                              "The raw arrow::flight::FlightInfo *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "info",
+    "Info",
+    "The raw arrow::flight::FlightInfo *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_INFO, spec);
 }
 
@@ -953,15 +938,12 @@ gaflight_info_new(GArrowSchema *schema,
     auto endpoint = GAFLIGHT_ENDPOINT(node->data);
     flight_endpoints.push_back(*gaflight_endpoint_get_raw(endpoint));
   }
-  auto flight_info_result =
-    arrow::flight::FlightInfo::Make(*arrow_schema,
-                                    *flight_descriptor,
-                                    flight_endpoints,
-                                    total_records,
-                                    total_bytes);
-  if (!garrow::check(error,
-                     flight_info_result,
-                     "[flight-info][new]")) {
+  auto flight_info_result = arrow::flight::FlightInfo::Make(*arrow_schema,
+                                                            *flight_descriptor,
+                                                            flight_endpoints,
+                                                            total_records,
+                                                            total_bytes);
+  if (!garrow::check(error, flight_info_result, "[flight-info][new]")) {
     return NULL;
   }
   return gaflight_info_new_raw(&(*flight_info_result));
@@ -978,22 +960,15 @@ gaflight_info_new(GArrowSchema *schema,
  * Since: 5.0.0
  */
 gboolean
-gaflight_info_equal(GAFlightInfo *info,
-                    GAFlightInfo *other_info)
+gaflight_info_equal(GAFlightInfo *info, GAFlightInfo *other_info)
 {
   const auto flight_info = gaflight_info_get_raw(info);
   const auto flight_other_info = gaflight_info_get_raw(other_info);
-  return
-    (flight_info->serialized_schema() ==
-     flight_other_info->serialized_schema()) &&
-    (flight_info->descriptor() ==
-     flight_other_info->descriptor()) &&
-    (flight_info->endpoints() ==
-     flight_other_info->endpoints()) &&
-    (flight_info->total_records() ==
-     flight_other_info->total_records()) &&
-    (flight_info->total_bytes() ==
-     flight_other_info->total_bytes());
+  return (flight_info->serialized_schema() == flight_other_info->serialized_schema()) &&
+         (flight_info->descriptor() == flight_other_info->descriptor()) &&
+         (flight_info->endpoints() == flight_other_info->endpoints()) &&
+         (flight_info->total_records() == flight_other_info->total_records()) &&
+         (flight_info->total_bytes() == flight_other_info->total_bytes());
 }
 
 /**
@@ -1007,19 +982,17 @@ gaflight_info_equal(GAFlightInfo *info,
  * Since: 5.0.0
  */
 GArrowSchema *
-gaflight_info_get_schema(GAFlightInfo *info,
-                         GArrowReadOptions *options,
-                         GError **error)
+gaflight_info_get_schema(GAFlightInfo *info, GArrowReadOptions *options, GError **error)
 {
   const auto flight_info = gaflight_info_get_raw(info);
   arrow::Status status;
   std::shared_ptr<arrow::Schema> arrow_schema;
   if (options) {
     auto arrow_memo = garrow_read_options_get_dictionary_memo_raw(options);
-    status = flight_info->GetSchema(arrow_memo, &arrow_schema);
+    status = flight_info->GetSchema(arrow_memo).Value(&arrow_schema);
   } else {
     arrow::ipc::DictionaryMemo arrow_memo;
-    status = flight_info->GetSchema(&arrow_memo, &arrow_schema);
+    status = flight_info->GetSchema(&arrow_memo).Value(&arrow_schema);
   }
   if (garrow::check(error, status, "[flight-info][get-schema]")) {
     return garrow_schema_new_raw(&arrow_schema);
@@ -1094,9 +1067,353 @@ gaflight_info_get_total_bytes(GAFlightInfo *info)
   return flight_info->total_bytes();
 }
 
+typedef struct GAFlightStreamChunkPrivate_
+{
+  arrow::flight::FlightStreamChunk chunk;
+} GAFlightStreamChunkPrivate;
+
+enum {
+  PROP_CHUNK = 1,
+};
+
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightStreamChunk, gaflight_stream_chunk, G_TYPE_OBJECT)
+
+#define GAFLIGHT_STREAM_CHUNK_GET_PRIVATE(obj)                                           \
+  static_cast<GAFlightStreamChunkPrivate *>(                                             \
+    gaflight_stream_chunk_get_instance_private(GAFLIGHT_STREAM_CHUNK(obj)))
+
+static void
+gaflight_stream_chunk_finalize(GObject *object)
+{
+  auto priv = GAFLIGHT_STREAM_CHUNK_GET_PRIVATE(object);
+
+  priv->chunk.~FlightStreamChunk();
+
+  G_OBJECT_CLASS(gaflight_info_parent_class)->finalize(object);
+}
+
+static void
+gaflight_stream_chunk_set_property(GObject *object,
+                                   guint prop_id,
+                                   const GValue *value,
+                                   GParamSpec *pspec)
+{
+  auto priv = GAFLIGHT_STREAM_CHUNK_GET_PRIVATE(object);
+
+  switch (prop_id) {
+  case PROP_CHUNK:
+    priv->chunk =
+      *static_cast<arrow::flight::FlightStreamChunk *>(g_value_get_pointer(value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+    break;
+  }
+}
+
+static void
+gaflight_stream_chunk_init(GAFlightStreamChunk *object)
+{
+}
+
+static void
+gaflight_stream_chunk_class_init(GAFlightStreamChunkClass *klass)
+{
+  auto gobject_class = G_OBJECT_CLASS(klass);
+
+  gobject_class->finalize = gaflight_stream_chunk_finalize;
+  gobject_class->set_property = gaflight_stream_chunk_set_property;
+
+  GParamSpec *spec;
+  spec = g_param_spec_pointer(
+    "chunk",
+    "Stream chunk",
+    "The raw arrow::flight::FlightStreamChunk *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property(gobject_class, PROP_CHUNK, spec);
+}
+
+/**
+ * gaflight_stream_chunk_get_data:
+ * @chunk: A #GAFlightStreamChunk.
+ *
+ * Returns: (transfer full): The data of the chunk.
+ *
+ * Since: 6.0.0
+ */
+GArrowRecordBatch *
+gaflight_stream_chunk_get_data(GAFlightStreamChunk *chunk)
+{
+  auto flight_chunk = gaflight_stream_chunk_get_raw(chunk);
+  return garrow_record_batch_new_raw(&(flight_chunk->data));
+}
+
+/**
+ * gaflight_stream_chunk_get_metadata:
+ * @chunk: A #GAFlightStreamChunk.
+ *
+ * Returns: (nullable) (transfer full): The metadata of the chunk.
+ *
+ *   The metadata may be NULL.
+ *
+ * Since: 6.0.0
+ */
+GArrowBuffer *
+gaflight_stream_chunk_get_metadata(GAFlightStreamChunk *chunk)
+{
+  auto flight_chunk = gaflight_stream_chunk_get_raw(chunk);
+  if (flight_chunk->app_metadata) {
+    return garrow_buffer_new_raw(&(flight_chunk->app_metadata));
+  } else {
+    return NULL;
+  }
+}
+
+typedef struct GAFlightRecordBatchReaderPrivate_
+{
+  arrow::flight::MetadataRecordBatchReader *reader;
+  bool is_owner;
+} GAFlightRecordBatchReaderPrivate;
+
+enum {
+  PROP_RECORD_BATCH_READER_READER = 1,
+  PROP_RECORD_BATCH_READER_IS_OWNER,
+};
+
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE(GAFlightRecordBatchReader,
+                                    gaflight_record_batch_reader,
+                                    G_TYPE_OBJECT)
+
+#define GAFLIGHT_RECORD_BATCH_READER_GET_PRIVATE(obj)                                    \
+  static_cast<GAFlightRecordBatchReaderPrivate *>(                                       \
+    gaflight_record_batch_reader_get_instance_private(                                   \
+      GAFLIGHT_RECORD_BATCH_READER(obj)))
+
+static void
+gaflight_record_batch_reader_finalize(GObject *object)
+{
+  auto priv = GAFLIGHT_RECORD_BATCH_READER_GET_PRIVATE(object);
+  if (priv->is_owner) {
+    delete priv->reader;
+  }
+  G_OBJECT_CLASS(gaflight_record_batch_reader_parent_class)->finalize(object);
+}
+
+static void
+gaflight_record_batch_reader_set_property(GObject *object,
+                                          guint prop_id,
+                                          const GValue *value,
+                                          GParamSpec *pspec)
+{
+  auto priv = GAFLIGHT_RECORD_BATCH_READER_GET_PRIVATE(object);
+
+  switch (prop_id) {
+  case PROP_RECORD_BATCH_READER_READER:
+    priv->reader =
+      static_cast<arrow::flight::MetadataRecordBatchReader *>(g_value_get_pointer(value));
+    break;
+  case PROP_RECORD_BATCH_READER_IS_OWNER:
+    priv->is_owner = g_value_get_boolean(value);
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+    break;
+  }
+}
+
+static void
+gaflight_record_batch_reader_init(GAFlightRecordBatchReader *object)
+{
+}
+
+static void
+gaflight_record_batch_reader_class_init(GAFlightRecordBatchReaderClass *klass)
+{
+  auto gobject_class = G_OBJECT_CLASS(klass);
+
+  gobject_class->finalize = gaflight_record_batch_reader_finalize;
+  gobject_class->set_property = gaflight_record_batch_reader_set_property;
+
+  GParamSpec *spec;
+  spec = g_param_spec_pointer(
+    "reader",
+    nullptr,
+    nullptr,
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property(gobject_class, PROP_RECORD_BATCH_READER_READER, spec);
+
+  spec = g_param_spec_boolean(
+    "is-owner",
+    nullptr,
+    nullptr,
+    TRUE,
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+  g_object_class_install_property(gobject_class, PROP_RECORD_BATCH_READER_IS_OWNER, spec);
+}
+
+/**
+ * gaflight_record_batch_reader_read_next:
+ * @reader: A #GAFlightRecordBatchReader.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (transfer full): The next chunk on success, %NULL on end
+ *   of stream, %NULL on error.
+ *
+ * Since: 6.0.0
+ */
+GAFlightStreamChunk *
+gaflight_record_batch_reader_read_next(GAFlightRecordBatchReader *reader, GError **error)
+{
+  auto flight_reader = gaflight_record_batch_reader_get_raw(reader);
+  arrow::flight::FlightStreamChunk flight_chunk;
+  auto status = flight_reader->Next().Value(&flight_chunk);
+  if (garrow::check(error, status, "[flight-record-batch-reader][read-next]")) {
+    if (flight_chunk.data) {
+      return gaflight_stream_chunk_new_raw(&flight_chunk);
+    } else {
+      return NULL;
+    }
+  } else {
+    return NULL;
+  }
+}
+
+/**
+ * gaflight_record_batch_reader_read_all:
+ * @reader: A #GAFlightRecordBatchReader.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (transfer full): The all data on success, %NULL on error.
+ *
+ * Since: 6.0.0
+ */
+GArrowTable *
+gaflight_record_batch_reader_read_all(GAFlightRecordBatchReader *reader, GError **error)
+{
+  auto flight_reader = gaflight_record_batch_reader_get_raw(reader);
+  std::shared_ptr<arrow::Table> arrow_table;
+  auto status = flight_reader->ToTable().Value(&arrow_table);
+  if (garrow::check(error, status, "[flight-record-batch-reader][read-all]")) {
+    return garrow_table_new_raw(&arrow_table);
+  } else {
+    return NULL;
+  }
+}
+
+G_DEFINE_ABSTRACT_TYPE(GAFlightRecordBatchWriter,
+                       gaflight_record_batch_writer,
+                       GARROW_TYPE_RECORD_BATCH_WRITER)
+
+static void
+gaflight_record_batch_writer_init(GAFlightRecordBatchWriter *object)
+{
+}
+
+static void
+gaflight_record_batch_writer_class_init(GAFlightRecordBatchWriterClass *klass)
+{
+}
+
+/**
+ * gaflight_record_batch_writer_begin:
+ * @writer: A #GAFlightRecordBatchWriter.
+ * @schema: A #GArrowSchema.
+ * @options: (nullable): A #GArrowWriteOptions.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Begins writing data with the given schema. Only used with
+ * `DoExchange`.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 18.0.0
+ */
+gboolean
+gaflight_record_batch_writer_begin(GAFlightRecordBatchWriter *writer,
+                                   GArrowSchema *schema,
+                                   GArrowWriteOptions *options,
+                                   GError **error)
+{
+  auto flight_writer = std::static_pointer_cast<arrow::flight::MetadataRecordBatchWriter>(
+    garrow_record_batch_writer_get_raw(GARROW_RECORD_BATCH_WRITER(writer)));
+  auto arrow_schema = garrow_schema_get_raw(schema);
+  arrow::ipc::IpcWriteOptions arrow_write_options;
+  if (options) {
+    arrow_write_options = *garrow_write_options_get_raw(options);
+  } else {
+    arrow_write_options = arrow::ipc::IpcWriteOptions::Defaults();
+  }
+  return garrow::check(error,
+                       flight_writer->Begin(arrow_schema, arrow_write_options),
+                       "[flight-record-batch-writer][begin]");
+}
+
+/**
+ * gaflight_record_batch_writer_write_metadata:
+ * @writer: A #GAFlightRecordBatchWriter.
+ * @metadata: A #GArrowBuffer.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Write metadata.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 18.0.0
+ */
+gboolean
+gaflight_record_batch_writer_write_metadata(GAFlightRecordBatchWriter *writer,
+                                            GArrowBuffer *metadata,
+                                            GError **error)
+{
+  auto flight_writer = std::static_pointer_cast<arrow::flight::MetadataRecordBatchWriter>(
+    garrow_record_batch_writer_get_raw(GARROW_RECORD_BATCH_WRITER(writer)));
+  auto arrow_metadata = garrow_buffer_get_raw(metadata);
+  return garrow::check(error,
+                       flight_writer->WriteMetadata(arrow_metadata),
+                       "[flight-record-batch-writer][write-metadata]");
+}
+
+/**
+ * gaflight_record_batch_writer_write_record_batch:
+ * @writer: A #GAFlightRecordBatchWriter.
+ * @record_batch: A #GArrowRecordBatch.
+ * @metadata: (nullable): A #GArrowBuffer.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Write a record batch with metadata.
+ *
+ * Returns: %TRUE on success, %FALSE on error.
+ *
+ * Since: 18.0.0
+ */
+gboolean
+gaflight_record_batch_writer_write_record_batch(GAFlightRecordBatchWriter *writer,
+                                                GArrowRecordBatch *record_batch,
+                                                GArrowBuffer *metadata,
+                                                GError **error)
+{
+  auto flight_writer = std::static_pointer_cast<arrow::flight::MetadataRecordBatchWriter>(
+    garrow_record_batch_writer_get_raw(GARROW_RECORD_BATCH_WRITER(writer)));
+  auto arrow_record_batch = garrow_record_batch_get_raw(record_batch);
+  auto arrow_metadata = garrow_buffer_get_raw(metadata);
+  return garrow::check(
+    error,
+    flight_writer->WriteWithMetadata(*arrow_record_batch, arrow_metadata),
+    "[flight-record-batch-writer][write]");
+}
 
 G_END_DECLS
 
+GAFlightCriteria *
+gaflight_criteria_new_raw(const arrow::flight::Criteria *flight_criteria)
+{
+  auto criteria = g_object_new(GAFLIGHT_TYPE_CRITERIA, NULL);
+  auto priv = GAFLIGHT_CRITERIA_GET_PRIVATE(criteria);
+  priv->criteria = *flight_criteria;
+  priv->expression =
+    g_bytes_new(priv->criteria.expression.data(), priv->criteria.expression.size());
+  return GAFLIGHT_CRITERIA(criteria);
+}
 
 arrow::flight::Criteria *
 gaflight_criteria_get_raw(GAFlightCriteria *criteria)
@@ -1113,8 +1430,7 @@ gaflight_location_get_raw(GAFlightLocation *location)
 }
 
 GAFlightDescriptor *
-gaflight_descriptor_new_raw(
-  const arrow::flight::FlightDescriptor *flight_descriptor)
+gaflight_descriptor_new_raw(const arrow::flight::FlightDescriptor *flight_descriptor)
 {
   GType gtype = GAFLIGHT_TYPE_DESCRIPTOR;
   switch (flight_descriptor->type) {
@@ -1127,9 +1443,7 @@ gaflight_descriptor_new_raw(
   default:
     break;
   }
-  return GAFLIGHT_DESCRIPTOR(g_object_new(gtype,
-                                          "descriptor", flight_descriptor,
-                                          NULL));
+  return GAFLIGHT_DESCRIPTOR(g_object_new(gtype, "descriptor", flight_descriptor, NULL));
 }
 
 arrow::flight::FlightDescriptor *
@@ -1139,6 +1453,16 @@ gaflight_descriptor_get_raw(GAFlightDescriptor *descriptor)
   return &(priv->descriptor);
 }
 
+GAFlightTicket *
+gaflight_ticket_new_raw(const arrow::flight::Ticket *flight_ticket)
+{
+  auto ticket = g_object_new(GAFLIGHT_TYPE_TICKET, NULL);
+  auto priv = GAFLIGHT_TICKET_GET_PRIVATE(ticket);
+  priv->ticket = *flight_ticket;
+  priv->data = g_bytes_new(priv->ticket.ticket.data(), priv->ticket.ticket.size());
+  return GAFLIGHT_TICKET(ticket);
+}
+
 arrow::flight::Ticket *
 gaflight_ticket_get_raw(GAFlightTicket *ticket)
 {
@@ -1150,8 +1474,7 @@ GAFlightEndpoint *
 gaflight_endpoint_new_raw(const arrow::flight::FlightEndpoint *flight_endpoint,
                           GAFlightTicket *ticket)
 {
-  auto endpoint = GAFLIGHT_ENDPOINT(g_object_new(GAFLIGHT_TYPE_ENDPOINT,
-                                                 NULL));
+  auto endpoint = GAFLIGHT_ENDPOINT(g_object_new(GAFLIGHT_TYPE_ENDPOINT, NULL));
   auto priv = GAFLIGHT_ENDPOINT_GET_PRIVATE(endpoint);
   if (ticket) {
     priv->ticket = ticket;
@@ -1181,9 +1504,7 @@ gaflight_endpoint_get_raw(GAFlightEndpoint *endpoint)
 GAFlightInfo *
 gaflight_info_new_raw(arrow::flight::FlightInfo *flight_info)
 {
-  return GAFLIGHT_INFO(g_object_new(GAFLIGHT_TYPE_INFO,
-                                    "info", flight_info,
-                                    NULL));
+  return GAFLIGHT_INFO(g_object_new(GAFLIGHT_TYPE_INFO, "info", flight_info, NULL));
 }
 
 arrow::flight::FlightInfo *
@@ -1192,3 +1513,24 @@ gaflight_info_get_raw(GAFlightInfo *info)
   auto priv = GAFLIGHT_INFO_GET_PRIVATE(info);
   return &(priv->info);
 }
+
+GAFlightStreamChunk *
+gaflight_stream_chunk_new_raw(arrow::flight::FlightStreamChunk *flight_chunk)
+{
+  return GAFLIGHT_STREAM_CHUNK(
+    g_object_new(GAFLIGHT_TYPE_STREAM_CHUNK, "chunk", flight_chunk, NULL));
+}
+
+arrow::flight::FlightStreamChunk *
+gaflight_stream_chunk_get_raw(GAFlightStreamChunk *chunk)
+{
+  auto priv = GAFLIGHT_STREAM_CHUNK_GET_PRIVATE(chunk);
+  return &(priv->chunk);
+}
+
+arrow::flight::MetadataRecordBatchReader *
+gaflight_record_batch_reader_get_raw(GAFlightRecordBatchReader *reader)
+{
+  auto priv = GAFLIGHT_RECORD_BATCH_READER_GET_PRIVATE(reader);
+  return priv->reader;
+}
