@@ -28,15 +28,15 @@ fi
 
 source_dir=${1}
 
-: ${ARROW_AZURE:=ON}
-: ${ARROW_FLIGHT:=ON}
-: ${ARROW_GCS:=ON}
-: ${ARROW_S3:=ON}
-: ${ARROW_SUBSTRAIT:=ON}
-: ${CHECK_IMPORTS:=ON}
-: ${CHECK_WHEEL_CONTENT:=ON}
-: ${CHECK_UNITTESTS:=ON}
-: ${INSTALL_PYARROW:=ON}
+: "${ARROW_AZURE:=ON}"
+: "${ARROW_FLIGHT:=ON}"
+: "${ARROW_GCS:=ON}"
+: "${ARROW_S3:=ON}"
+: "${ARROW_SUBSTRAIT:=ON}"
+: "${CHECK_IMPORTS:=ON}"
+: "${CHECK_WHEEL_CONTENT:=ON}"
+: "${CHECK_UNITTESTS:=ON}"
+: "${INSTALL_PYARROW:=ON}"
 
 export PYARROW_TEST_ACERO=ON
 export PYARROW_TEST_AZURE=${ARROW_AZURE}
@@ -59,7 +59,7 @@ export PARQUET_TEST_DATA=${source_dir}/cpp/submodules/parquet-testing/data
 
 if [ "${INSTALL_PYARROW}" == "ON" ]; then
   # Install the built wheels
-  python -m pip install ${source_dir}/python/repaired_wheels/*.whl
+  python -m pip install "${source_dir}"/python/repaired_wheels/*.whl
 fi
 
 if [ "${CHECK_IMPORTS}" == "ON" ]; then
@@ -96,13 +96,13 @@ if [ "${CHECK_VERSION}" == "ON" ]; then
 fi
 
 if [ "${CHECK_WHEEL_CONTENT}" == "ON" ]; then
-  python ${source_dir}/ci/scripts/python_wheel_validate_contents.py \
-    --path ${source_dir}/python/repaired_wheels
+  python "${source_dir}/ci/scripts/python_wheel_validate_contents.py" \
+    --path "${source_dir}/python/repaired_wheels"
 fi
 
 if [ "${CHECK_UNITTESTS}" == "ON" ]; then
   # Install testing dependencies
-  python -m pip install -U -r ${source_dir}/python/requirements-wheel-test.txt
+  python -m pip install -U -r "${source_dir}/python/requirements-wheel-test.txt"
 
   # Execute unittest, test dependencies must be installed
   python -c 'import pyarrow; pyarrow.create_library_symlinks()'
