@@ -15,12 +15,14 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include <gmock/gmock-matchers.h>
 #include <gtest/gtest.h>
 
 #include <orc/OrcFile.hh>
 #include <string>
 
 #include "arrow/adapters/orc/adapter.h"
+#include "arrow/adapters/orc/util.h"
 #include "arrow/array.h"
 #include "arrow/buffer.h"
 #include "arrow/buffer_builder.h"
@@ -31,9 +33,10 @@
 #include "arrow/status.h"
 #include "arrow/table.h"
 #include "arrow/testing/gtest_util.h"
+#include "arrow/testing/matchers.h"
 #include "arrow/testing/random.h"
 #include "arrow/type.h"
-#include "arrow/util/decimal.h"
+#include "arrow/util/io_util.h"
 #include "arrow/util/key_value_metadata.h"
 
 namespace liborc = orc;
@@ -80,7 +83,8 @@ class MemoryOutputStream : public liborc::OutputStream {
  private:
   std::vector<char> data_;
   std::string name_;
-  uint64_t length_, natural_write_size_;
+  uint64_t length_;
+  const uint64_t natural_write_size_ = 64 * 1024;
 };
 
 std::shared_ptr<Buffer> GenerateFixedDifferenceBuffer(int32_t fixed_length,
@@ -123,30 +127,25 @@ Result<std::shared_ptr<Array>> GenerateRandomDate64Array(int64_t size,
       date64(), rand.Int64(size, kMilliMin, kMilliMax, null_probability));
 }
 
-Result<std::shared_ptr<Array>> GenerateRandomTimestampArray(int64_t size,
-                                                            TimeUnit::type type,
-                                                            double null_probability) {
+Result<std::shared_ptr<Array>> GenerateRandomTimestampArray(
+    int64_t size, std::shared_ptr<TimestampType> type, double null_probability) {
   random::RandomArrayGenerator rand(kRandomSeed);
-  switch (type) {
+  switch (type->unit()) {
     case TimeUnit::type::SECOND: {
       return CastInt64ArrayToTemporalArray<TimestampArray>(
-          timestamp(TimeUnit::SECOND),
-          rand.Int64(size, kSecondMin, kSecondMax, null_probability));
+          type, rand.Int64(size, kSecondMin, kSecondMax, null_probability));
     }
     case TimeUnit::type::MILLI: {
       return CastInt64ArrayToTemporalArray<TimestampArray>(
-          timestamp(TimeUnit::MILLI),
-          rand.Int64(size, kMilliMin, kMilliMax, null_probability));
+          type, rand.Int64(size, kMilliMin, kMilliMax, null_probability));
     }
     case TimeUnit::type::MICRO: {
       return CastInt64ArrayToTemporalArray<TimestampArray>(
-          timestamp(TimeUnit::MICRO),
-          rand.Int64(size, kMicroMin, kMicroMax, null_probability));
+          type, rand.Int64(size, kMicroMin, kMicroMax, null_probability));
     }
     case TimeUnit::type::NANO: {
       return CastInt64ArrayToTemporalArray<TimestampArray>(
-          timestamp(TimeUnit::NANO),
-          rand.Int64(size, kNanoMin, kNanoMax, null_probability));
+          type, rand.Int64(size, kNanoMin, kNanoMax, null_probability));
     }
     default: {
       return Status::TypeError("Unknown or unsupported Arrow TimeUnit: ", type);
@@ -195,12 +194,11 @@ std::shared_ptr<ChunkedArray> GenerateRandomChunkedArray(
         break;
       }
       case Type::TIMESTAMP: {
-        EXPECT_OK_AND_ASSIGN(
-            arrays[j],
-            GenerateRandomTimestampArray(
-                current_size_chunks[j],
-                internal::checked_pointer_cast<TimestampType>(data_type)->unit(),
-                null_probability));
+        EXPECT_OK_AND_ASSIGN(arrays[j],
+                             GenerateRandomTimestampArray(
+                                 current_size_chunks[j],
+                                 internal::checked_pointer_cast<TimestampType>(data_type),
+                                 null_probability));
         break;
       }
       default:
@@ -224,7 +222,7 @@ std::shared_ptr<Table> GenerateRandomTable(const std::shared_ptr<Schema>& schema
   return Table::Make(schema, cv);
 }
 
-void AssertTableWriteReadEqual(const std::shared_ptr<Table>& input_table,
+void AssertTableWriteReadEqual(const std::vector<std::shared_ptr<Table>>& input_tables,
                                const std::shared_ptr<Table>& expected_output_table,
                                const int64_t max_size = kDefaultSmallMemStreamSize,
                                std::vector<int>* opt_selected_read_indices = nullptr) {
@@ -237,11 +235,13 @@ void AssertTableWriteReadEqual(const std::shared_ptr<Table>& input_table,
   write_options.compression = Compression::UNCOMPRESSED;
 #endif
   write_options.file_version = adapters::orc::FileVersion(0, 11);
-  write_options.compression_block_size = 32768;
+  write_options.compression_block_size = 64 * 1024;
   write_options.row_index_stride = 5000;
   EXPECT_OK_AND_ASSIGN(auto writer, adapters::orc::ORCFileWriter::Open(
                                         buffer_output_stream.get(), write_options));
-  ARROW_EXPECT_OK(writer->Write(*input_table));
+  for (const auto& input_table : input_tables) {
+    ARROW_EXPECT_OK(writer->Write(*input_table));
+  }
   ARROW_EXPECT_OK(writer->Close());
   EXPECT_OK_AND_ASSIGN(auto buffer, buffer_output_stream->Finish());
   std::shared_ptr<io::RandomAccessFile> in_stream(new io::BufferReader(buffer));
@@ -259,6 +259,48 @@ void AssertTableWriteReadEqual(const std::shared_ptr<Table>& input_table,
   AssertTablesEqual(*expected_output_table, *actual_output_table, false, false);
 }
 
+void AssertBatchWriteReadEqual(
+    const std::vector<std::shared_ptr<RecordBatch>>& input_batches,
+    const std::shared_ptr<Table>& expected_output_table,
+    const int64_t max_size = kDefaultSmallMemStreamSize) {
+  EXPECT_OK_AND_ASSIGN(auto buffer_output_stream,
+                       io::BufferOutputStream::Create(max_size));
+  auto write_options = adapters::orc::WriteOptions();
+#ifdef ARROW_WITH_SNAPPY
+  write_options.compression = Compression::SNAPPY;
+#else
+  write_options.compression = Compression::UNCOMPRESSED;
+#endif
+  write_options.file_version = adapters::orc::FileVersion(0, 11);
+  write_options.compression_block_size = 64 * 1024;
+  write_options.row_index_stride = 5000;
+  EXPECT_OK_AND_ASSIGN(auto writer, adapters::orc::ORCFileWriter::Open(
+                                        buffer_output_stream.get(), write_options));
+  for (auto& input_batch : input_batches) {
+    ARROW_EXPECT_OK(writer->Write(*input_batch));
+  }
+  ARROW_EXPECT_OK(writer->Close());
+  EXPECT_OK_AND_ASSIGN(auto buffer, buffer_output_stream->Finish());
+  std::shared_ptr<io::RandomAccessFile> in_stream(new io::BufferReader(buffer));
+  EXPECT_OK_AND_ASSIGN(
+      auto reader, adapters::orc::ORCFileReader::Open(in_stream, default_memory_pool()));
+  ASSERT_EQ(reader->GetFileVersion(), write_options.file_version);
+  ASSERT_EQ(reader->GetCompression(), write_options.compression);
+  ASSERT_EQ(reader->GetCompressionSize(), write_options.compression_block_size);
+  ASSERT_EQ(reader->GetRowIndexStride(), write_options.row_index_stride);
+  EXPECT_OK_AND_ASSIGN(auto actual_output_table, reader->Read());
+  AssertTablesEqual(*expected_output_table, *actual_output_table, false, false);
+}
+
+void AssertTableWriteReadEqual(const std::shared_ptr<Table>& input_table,
+                               const std::shared_ptr<Table>& expected_output_table,
+                               const int64_t max_size = kDefaultSmallMemStreamSize,
+                               std::vector<int>* opt_selected_read_indices = nullptr) {
+  std::vector<std::shared_ptr<Table>> input_tables;
+  input_tables.push_back(input_table);
+  AssertTableWriteReadEqual(input_tables, expected_output_table, max_size,
+                            opt_selected_read_indices);
+}
 void AssertArrayWriteReadEqual(const std::shared_ptr<Array>& input_array,
                                const std::shared_ptr<Array>& expected_output_array,
                                const int64_t max_size = kDefaultSmallMemStreamSize) {
@@ -288,7 +330,7 @@ std::unique_ptr<liborc::Writer> CreateWriter(uint64_t stripe_size,
                                              liborc::OutputStream* stream) {
   liborc::WriterOptions options;
   options.setStripeSize(stripe_size);
-  options.setCompressionBlockSize(1024);
+  options.setCompressionBlockSize(64 * 1024);
   options.setMemoryPool(liborc::getDefaultPool());
   options.setRowIndexStride(0);
   return liborc::createWriter(type, stream, options);
@@ -296,7 +338,7 @@ std::unique_ptr<liborc::Writer> CreateWriter(uint64_t stripe_size,
 
 TEST(TestAdapterRead, ReadIntAndStringFileMultipleStripes) {
   MemoryOutputStream mem_stream(kDefaultMemStreamSize);
-  ORC_UNIQUE_PTR<liborc::Type> type(
+  std::unique_ptr<liborc::Type> type(
       liborc::Type::buildTypeFromString("struct<col1:int,col2:string>"));
 
   constexpr uint64_t stripe_size = 1024;  // 1K
@@ -347,6 +389,10 @@ TEST(TestAdapterRead, ReadIntAndStringFileMultipleStripes) {
   ASSERT_TRUE(metadata->Equals(*expected_metadata));
   ASSERT_EQ(stripe_row_count * stripe_count, reader->NumberOfRows());
   ASSERT_EQ(stripe_count, reader->NumberOfStripes());
+  ASSERT_EQ(static_cast<int64_t>(stripe_row_count),
+            reader->GetStripeInformation(0).num_rows);
+  ASSERT_EQ(static_cast<int64_t>(reader->NumberOfRows() - stripe_row_count),
+            reader->GetStripeInformation(stripe_count - 1).first_row_id);
   accumulated = 0;
   EXPECT_OK_AND_ASSIGN(auto stripe_reader, reader->NextStripeReader(reader_batch_size));
   while (stripe_reader) {
@@ -410,6 +456,205 @@ TEST(TestAdapterRead, ReadIntAndStringFileMultipleStripes) {
   EXPECT_EQ(num_rows / reader_batch_size, batches);
 }
 
+TEST(TestAdapterRead, ReadCharAndVarcharType) {
+  MemoryOutputStream mem_stream(kDefaultMemStreamSize);
+  auto orc_type = liborc::Type::buildTypeFromString("struct<c1:char(6),c2:varchar(6)>");
+  auto writer = CreateWriter(/*stripe_size=*/1024, *orc_type, &mem_stream);
+
+  constexpr int64_t row_count = 2;
+  auto batch = writer->createRowBatch(row_count);
+  auto struct_batch = internal::checked_cast<liborc::StructVectorBatch*>(batch.get());
+
+  // Verify that longer data will be truncated by ORC char and varchar types.
+  // In addition, ORC char type will pad the data with spaces if the data is shorter.
+  const std::vector<std::string> data = {"abcd", "ABCDEFGH"};
+  const std::vector<std::vector<std::string>> expected_data = {{"abcd  ", "ABCDEF"},
+                                                               {"abcd", "ABCDEF"}};
+
+  for (uint64_t col = 0; col < orc_type->getSubtypeCount(); ++col) {
+    auto str_batch =
+        internal::checked_cast<liborc::StringVectorBatch*>(struct_batch->fields[col]);
+    str_batch->hasNulls = false;
+    str_batch->numElements = row_count;
+    for (int64_t row = 0; row < row_count; ++row) {
+      str_batch->data[row] = const_cast<char*>(data[row].c_str());
+      str_batch->length[row] = static_cast<int64_t>(data[row].size());
+    }
+  }
+  batch->numElements = row_count;
+  writer->add(*batch);
+  writer->close();
+
+  std::shared_ptr<io::RandomAccessFile> in_stream = std::make_shared<io::BufferReader>(
+      std::make_shared<Buffer>(reinterpret_cast<const uint8_t*>(mem_stream.getData()),
+                               mem_stream.getLength()));
+  ASSERT_OK_AND_ASSIGN(
+      auto reader, adapters::orc::ORCFileReader::Open(in_stream, default_memory_pool()));
+  ASSERT_EQ(row_count, reader->NumberOfRows());
+  ASSERT_EQ(1, reader->NumberOfStripes());
+
+  EXPECT_OK_AND_ASSIGN(auto stripe_reader, reader->NextStripeReader(row_count));
+  std::shared_ptr<RecordBatch> record_batch;
+  ASSERT_OK(stripe_reader->ReadNext(&record_batch));
+  ASSERT_NE(nullptr, record_batch);
+  ASSERT_EQ(row_count, record_batch->num_rows());
+
+  for (int col = 0; col < record_batch->num_columns(); ++col) {
+    auto str_array = checked_pointer_cast<StringArray>(record_batch->column(col));
+    for (int row = 0; row < row_count; ++row) {
+      EXPECT_EQ(expected_data[col][row], str_array->GetString(row));
+    }
+  }
+  ASSERT_OK(stripe_reader->ReadNext(&record_batch));
+  ASSERT_EQ(nullptr, record_batch);
+}
+
+TEST(TestAdapterRead, ReadFieldAttributes) {
+  const std::string id_key = "iceberg.id";
+  const std::string required_key = "iceberg.required";
+
+  auto set_attributes = [&](liborc::Type* type, const std::string& id,
+                            const std::string& required) {
+    type->setAttribute(id_key, id);
+    type->setAttribute(required_key, required);
+  };
+
+  auto check_attributes = [&](const std::shared_ptr<arrow::Field>& field,
+                              const std::string& expect_id,
+                              const std::string& expect_required) {
+    auto field_metadata = field->metadata();
+    ASSERT_NE(field_metadata, nullptr);
+    ASSERT_EQ(expect_id, field_metadata->Get(id_key));
+    ASSERT_EQ(expect_required, field_metadata->Get(required_key));
+  };
+
+  auto c1_type = liborc::createPrimitiveType(liborc::TypeKind::INT);
+  set_attributes(c1_type.get(), "1", "true");
+
+  auto c2_elem_type = liborc::createPrimitiveType(liborc::TypeKind::INT);
+  set_attributes(c2_elem_type.get(), "3", "false");
+  auto c2_type = liborc::createListType(std::move(c2_elem_type));
+  set_attributes(c2_type.get(), "2", "false");
+
+  auto c3_key_type = liborc::createPrimitiveType(liborc::TypeKind::INT);
+  set_attributes(c3_key_type.get(), "5", "true");
+  auto c3_value_type = liborc::createPrimitiveType(liborc::TypeKind::INT);
+  set_attributes(c3_value_type.get(), "6", "false");
+  auto c3_type = liborc::createMapType(std::move(c3_key_type), std::move(c3_value_type));
+  set_attributes(c3_type.get(), "4", "false");
+
+  auto c4_sub_type = liborc::createPrimitiveType(liborc::TypeKind::INT);
+  set_attributes(c4_sub_type.get(), "8", "false");
+  auto c4_type = liborc::createStructType();
+  c4_type->addStructField("c4_1", std::move(c4_sub_type));
+  set_attributes(c4_type.get(), "7", "false");
+
+  auto orc_type = liborc::createStructType();
+  orc_type->addStructField("c1", std::move(c1_type));
+  orc_type->addStructField("c2", std::move(c2_type));
+  orc_type->addStructField("c3", std::move(c3_type));
+  orc_type->addStructField("c4", std::move(c4_type));
+
+  MemoryOutputStream mem_stream(kDefaultMemStreamSize);
+  auto writer = CreateWriter(/*stripe_size=*/1024, *orc_type, &mem_stream);
+  writer->close();
+
+  std::shared_ptr<io::RandomAccessFile> in_stream = std::make_shared<io::BufferReader>(
+      std::make_shared<Buffer>(reinterpret_cast<const uint8_t*>(mem_stream.getData()),
+                               mem_stream.getLength()));
+  ASSERT_OK_AND_ASSIGN(
+      auto reader, adapters::orc::ORCFileReader::Open(in_stream, default_memory_pool()));
+  ASSERT_EQ(0, reader->NumberOfRows());
+
+  ASSERT_OK_AND_ASSIGN(auto schema, reader->ReadSchema());
+  ASSERT_EQ(4, schema->num_fields());
+
+  // check top level fields
+  check_attributes(schema->field(0), "1", "true");
+  check_attributes(schema->field(1), "2", "false");
+  check_attributes(schema->field(2), "4", "false");
+  check_attributes(schema->field(3), "7", "false");
+
+  // check list element type
+  auto list_type = checked_pointer_cast<arrow::ListType>(schema->field(1)->type());
+  check_attributes(list_type->value_field(), "3", "false");
+
+  // check map key/value types
+  auto map_type = checked_pointer_cast<arrow::MapType>(schema->field(2)->type());
+  check_attributes(map_type->key_field(), "5", "true");
+  check_attributes(map_type->item_field(), "6", "false");
+
+  // check struct sub-field type
+  auto struct_type = checked_pointer_cast<arrow::StructType>(schema->field(3)->type());
+  check_attributes(struct_type->field(0), "8", "false");
+}
+
+TEST(TestAdapterReadWrite, FieldAttributesRoundTrip) {
+  EXPECT_OK_AND_ASSIGN(auto buffer_output_stream, io::BufferOutputStream::Create(1024));
+  auto write_options = adapters::orc::WriteOptions();
+  write_options.compression = Compression::UNCOMPRESSED;
+  EXPECT_OK_AND_ASSIGN(auto writer, adapters::orc::ORCFileWriter::Open(
+                                        buffer_output_stream.get(), write_options));
+
+  auto schema = ::arrow::schema(
+      {::arrow::field("c0", ::arrow::int64(), /*nullable=*/true,
+                      key_value_metadata({"k0"}, {"v0"})),
+       ::arrow::field("c1", ::arrow::utf8(), /*nullable=*/true,
+                      key_value_metadata({"k1"}, {"v1"})),
+       ::arrow::field(
+           "c2", ::arrow::list(::arrow::field("item", ::arrow::int64(), /*nullable=*/true,
+                                              key_value_metadata({"k2"}, {"v2"})))),
+       ::arrow::field("c3",
+                      std::make_shared<MapType>(
+                          ::arrow::field("key", ::arrow::utf8(), /*nullable=*/false,
+                                         key_value_metadata({"k3"}, {"v3"})),
+                          ::arrow::field("value", ::arrow::int64(), /*nullable=*/true,
+                                         key_value_metadata({"k4"}, {"v4"})))),
+       ::arrow::field("c4", ::arrow::struct_({::arrow::field(
+                                "sub", ::arrow::int64(),
+                                /*nullable=*/true, key_value_metadata({"k5"}, {"v5"}))})),
+       ::arrow::field("c5",
+                      ::arrow::sparse_union(
+                          {::arrow::field("_union_0", ::arrow::int64(), /*nullable=*/true,
+                                          key_value_metadata({"k6"}, {"v6"})),
+                           ::arrow::field("_union_1", ::arrow::utf8(), /*nullable=*/true,
+                                          key_value_metadata({"k7"}, {"v7"}))},
+                          {0, 1}))});
+  auto expected_output_table = ::arrow::TableFromJSON(
+      schema, {R"([[1, "a", [1, 2], [["a", 1]], {"sub": 1}, null]])"});
+  ARROW_EXPECT_OK(writer->Write(*expected_output_table));
+  ARROW_EXPECT_OK(writer->Close());
+
+  EXPECT_OK_AND_ASSIGN(auto buffer, buffer_output_stream->Finish());
+  std::shared_ptr<io::RandomAccessFile> in_stream(new io::BufferReader(buffer));
+  EXPECT_OK_AND_ASSIGN(
+      auto reader, adapters::orc::ORCFileReader::Open(in_stream, default_memory_pool()));
+  EXPECT_OK_AND_ASSIGN(auto actual_output_table, reader->Read());
+  ASSERT_OK(actual_output_table->ValidateFull());
+  AssertTablesEqual(*expected_output_table, *actual_output_table);
+
+  // Check schema equality with metadata.
+  EXPECT_OK_AND_ASSIGN(auto read_schema, reader->ReadSchema());
+  AssertSchemaEqual(schema, read_schema, /*check_metadata=*/true);
+}
+
+TEST(TestAdapterReadWrite, ThrowWhenTZDBUnavaiable) {
+  if (adapters::orc::GetOrcMajorVersion() >= 2) {
+    GTEST_SKIP() << "Only ORC pre-2.0.0 versions have the time zone database check";
+  }
+
+  EnvVarGuard tzdir_guard("TZDIR", "/wrong/path");
+  const char* expect_str = "IANA time zone database is unavailable but required by ORC";
+  EXPECT_OK_AND_ASSIGN(auto out_stream, io::BufferOutputStream::Create(1024));
+  EXPECT_THAT(
+      adapters::orc::ORCFileWriter::Open(out_stream.get(), adapters::orc::WriteOptions()),
+      Raises(StatusCode::Invalid, testing::HasSubstr(expect_str)));
+  EXPECT_OK_AND_ASSIGN(auto buffer, out_stream->Finish());
+  EXPECT_THAT(adapters::orc::ORCFileReader::Open(
+                  std::make_shared<io::BufferReader>(buffer), default_memory_pool()),
+              Raises(StatusCode::Invalid, testing::HasSubstr(expect_str)));
+}
+
 // Trivial
 
 class TestORCWriterTrivialNoWrite : public ::testing::Test {};
@@ -423,7 +668,7 @@ TEST_F(TestORCWriterTrivialNoWrite, noWrite) {
   write_options.compression = Compression::UNCOMPRESSED;
 #endif
   write_options.file_version = adapters::orc::FileVersion(0, 11);
-  write_options.compression_block_size = 32768;
+  write_options.compression_block_size = 64 * 1024;
   write_options.row_index_stride = 5000;
   EXPECT_OK_AND_ASSIGN(auto writer, adapters::orc::ORCFileWriter::Open(
                                         buffer_output_stream.get(), write_options));
@@ -437,7 +682,8 @@ class TestORCWriterTrivialNoConversion : public ::testing::Test {
          field("int32", int32()), field("int64", int64()), field("float", float32()),
          field("double", float64()), field("decimal128nz", decimal128(25, 6)),
          field("decimal128z", decimal128(32, 0)), field("date32", date32()),
-         field("ts3", timestamp(TimeUnit::NANO)), field("string", utf8()),
+         field("ts3", timestamp(TimeUnit::NANO)),
+         field("ts4", timestamp(TimeUnit::NANO, "UTC")), field("string", utf8()),
          field("binary", binary()),
          field("struct", struct_({field("a", utf8()), field("b", int64())})),
          field("list", list(int32())),
@@ -477,6 +723,7 @@ TEST_F(TestORCWriterTrivialNoConversion, writeFilledChunkAndSelectField) {
       field("double", float64()),
       field("date32", date32()),
       field("ts3", timestamp(TimeUnit::NANO)),
+      field("ts4", timestamp(TimeUnit::NANO), "UTC"),
       field("string", utf8()),
       field("binary", binary()),
   });
@@ -487,6 +734,15 @@ TEST_F(TestORCWriterTrivialNoConversion, writeFilledChunkAndSelectField) {
                             &selected_indices);
 }
 
+TEST_F(TestORCWriterTrivialNoConversion, writeSlicedBatch) {
+  std::shared_ptr<Table> table =
+      GenerateRandomTable(table_schema, /*size=*/100, /*min_num_chunks=*/1,
+                          /*max_num_chunks*/ 1, /*probability=*/0);
+  table = table->Slice(20, 60);
+
+  AssertTableWriteReadEqual(table, table, kDefaultSmallMemStreamSize / 16);
+}
+
 class TestORCWriterTrivialWithConversion : public ::testing::Test {
  public:
   TestORCWriterTrivialWithConversion() {
@@ -524,6 +780,21 @@ TEST_F(TestORCWriterTrivialWithConversion, writeChunkless) {
                             kDefaultSmallMemStreamSize / 16);
 }
 
+class TestORCWriterInvalidTypes : public ::testing::Test {};
+
+TEST_F(TestORCWriterInvalidTypes, noWriteInvalidTypes) {
+  // Unsigned integers are not supported by ORC
+  std::shared_ptr<arrow::Schema> table_schema = schema({field("uint64", uint64())});
+  const std::shared_ptr<Table> table = GenerateRandomTable(table_schema, 100, 1, 1, 0);
+  EXPECT_OK_AND_ASSIGN(auto buffer_output_stream,
+                       io::BufferOutputStream::Create(kDefaultSmallMemStreamSize / 16));
+  EXPECT_OK_AND_ASSIGN(auto writer,
+                       adapters::orc::ORCFileWriter::Open(buffer_output_stream.get()));
+  EXPECT_RAISES_WITH_MESSAGE_THAT(NotImplemented,
+                                  testing::HasSubstr("Unknown or unsupported Arrow type"),
+                                  writer->Write(*table));
+}
+
 // General
 
 class TestORCWriterNoConversion : public ::testing::Test {
@@ -767,4 +1038,142 @@ TEST_F(TestORCWriterSingleArray, WriteListOfMap) {
   AssertArrayWriteReadEqual(array, array, kDefaultSmallMemStreamSize * 10);
 }
 
+namespace {
+
+// The random array generator fills random unselected values in the child arrays
+// of SparseUnionArray. However, orc file only contains dense union type meaning
+// that these unselected values will not be written to the file (so we can never
+// read them back and compare equality in the unit test). Because the orc reader
+// fills unselected values to nulls when reading from the file. So flattening
+// the SparseUnionArray before writing makes it easy for the array equality check.
+std::shared_ptr<Array> FlattenSparseUnionArray(std::shared_ptr<Array> array) {
+  auto union_array = checked_pointer_cast<SparseUnionArray>(array);
+  ArrayVector children;
+  for (int i = 0; i < array->num_fields(); ++i) {
+    ASSIGN_OR_ABORT(auto flattened_child, union_array->GetFlattenedField(i));
+    children.emplace_back(std::move(flattened_child));
+  }
+  return std::make_shared<SparseUnionArray>(array->type(), array->length(),
+                                            std::move(children),
+                                            union_array->type_codes(), array->offset());
+}
+
+void TestUnionConversion(std::shared_ptr<Array> array) {
+  auto length = array->length();
+  auto orc_type = liborc::Type::buildTypeFromString("uniontype<string,int>");
+
+  // Workaround for an unfortunate breaking change introduced by ORC-1.9.0.
+  MemoryOutputStream mem_stream(/*capacity=*/1024);
+  auto writer = CreateWriter(/*stripe_size=*/1024, *orc_type, &mem_stream);
+  auto orc_batch = writer->createRowBatch(length);
+  // auto orc_batch = orc_type->createRowBatch(length, *liborc::getDefaultPool());
+
+  // Convert from arrow to orc
+  int arrow_chunk_offset = 0;
+  int64_t arrow_index_offset = 0;
+  ASSERT_OK(adapters::orc::WriteBatch(ChunkedArray{array}, length, &arrow_chunk_offset,
+                                      &arrow_index_offset, orc_batch.get()));
+
+  // Convert from orc to arrow
+  ASSIGN_OR_ABORT(auto array_builder, MakeBuilder(array->type()));
+  ASSERT_OK(adapters::orc::AppendBatch(orc_type.get(), orc_batch.get(), 0, length,
+                                       array_builder.get()));
+  std::shared_ptr<Array> result;
+  ASSERT_OK(array_builder->Finish(&result));
+
+  // Compare the result
+  AssertArraysEqual(*array, *result);
+}
+
+}  // namespace
+
+TEST_F(TestORCWriterSingleArray, WriteSparseUnion) {
+  constexpr int64_t num_rows = 1024;
+  auto type =
+      sparse_union({field("_union_0", utf8()), field("_union_1", int32())}, {0, 1});
+  auto array =
+      FlattenSparseUnionArray(rand.ArrayOf(type, num_rows, /*null_probability=*/0.4));
+  AssertArrayWriteReadEqual(array, array, kDefaultSmallMemStreamSize * 10);
+}
+
+TEST(TestWriteReadORCBatch, SparseUnionConversion) {
+  random::RandomArrayGenerator rand(kRandomSeed);
+  auto type = sparse_union({field("a", utf8()), field("b", int32())}, {0, 1});
+  auto array = FlattenSparseUnionArray(
+      rand.ArrayOf(type, /*size=*/1024, /*null_probability=*/0.4));
+  TestUnionConversion(std::move(array));
+}
+
+TEST(TestWriteReadORCBatch, DenseUnionConversion) {
+  random::RandomArrayGenerator rand(kRandomSeed);
+  auto type = dense_union({field("a", utf8()), field("b", int32())}, {0, 1});
+  auto array = rand.ArrayOf(type, /*size=*/1024, /*null_probability=*/0.4);
+  TestUnionConversion(std::move(array));
+}
+
+class TestORCWriterMultipleWrite : public ::testing::Test {
+ public:
+  TestORCWriterMultipleWrite() : rand(kRandomSeed) {}
+
+ protected:
+  random::RandomArrayGenerator rand;
+};
+
+TEST_F(TestORCWriterMultipleWrite, MultipleWritesIntField) {
+  const int64_t num_rows = 1234;
+  const int num_writes = 5;
+  std::shared_ptr<Schema> input_schema = schema({field("col0", int32())});
+  ArrayVector vect;
+  std::vector<std::shared_ptr<Table>> input_tables;
+  for (int i = 0; i < num_writes; i++) {
+    auto array_int = rand.ArrayOf(int32(), num_rows, 0);
+    vect.push_back(array_int);
+    auto input_chunked_array = std::make_shared<ChunkedArray>(array_int);
+    input_tables.emplace_back(Table::Make(input_schema, {input_chunked_array}));
+  }
+  auto expected_output_chunked_array = std::make_shared<ChunkedArray>(vect);
+  std::shared_ptr<Table> expected_output_table =
+      Table::Make(input_schema, {expected_output_chunked_array});
+  AssertTableWriteReadEqual(input_tables, expected_output_table,
+                            kDefaultSmallMemStreamSize * 100);
+}
+
+TEST_F(TestORCWriterMultipleWrite, MultipleWritesIncoherentSchema) {
+  const int64_t num_rows = 1234;
+  auto array_int = rand.ArrayOf(int32(), num_rows, 0);
+  std::shared_ptr<Schema> input_schema = schema({field("col0", array_int->type())});
+  auto array_int2 = rand.ArrayOf(int64(), num_rows, 0);
+  std::shared_ptr<Schema> input_schema2 = schema({field("col0", array_int2->type())});
+
+  std::shared_ptr<Table> input_table = Table::Make(input_schema, {array_int});
+  std::shared_ptr<Table> input_table2 = Table::Make(input_schema2, {array_int2});
+  EXPECT_OK_AND_ASSIGN(auto buffer_output_stream,
+                       io::BufferOutputStream::Create(kDefaultSmallMemStreamSize));
+  auto write_options = adapters::orc::WriteOptions();
+  EXPECT_OK_AND_ASSIGN(auto writer, adapters::orc::ORCFileWriter::Open(
+                                        buffer_output_stream.get(), write_options));
+  ARROW_EXPECT_OK(writer->Write(*input_table));
+
+  // This should not pass
+  ASSERT_RAISES(TypeError, writer->Write(*input_table2));
+
+  ARROW_EXPECT_OK(writer->Close());
+}
+TEST_F(TestORCWriterMultipleWrite, MultipleWritesIntFieldRecordBatch) {
+  const int64_t num_rows = 1234;
+  const int num_writes = 5;
+  std::shared_ptr<Schema> input_schema = schema({field("col0", int32())});
+  ArrayVector vect;
+  std::vector<std::shared_ptr<RecordBatch>> input_batches;
+  for (int i = 0; i < num_writes; i++) {
+    auto array_int = rand.ArrayOf(int32(), num_rows, 0);
+    vect.push_back(array_int);
+    input_batches.emplace_back(RecordBatch::Make(input_schema, num_rows, {array_int}));
+  }
+  auto expected_output_chunked_array = std::make_shared<ChunkedArray>(vect);
+  std::shared_ptr<Table> expected_output_table =
+      Table::Make(input_schema, {expected_output_chunked_array});
+  AssertBatchWriteReadEqual(input_batches, expected_output_table,
+                            kDefaultSmallMemStreamSize * 100);
+}
 }  // namespace arrow
