@@ -24,13 +24,16 @@
 #include <cstdint>
 #include <cstring>
 #include <memory>
+#include <optional>
 #include <string>
 #include <type_traits>
 #include <utility>
+#include <variant>
 #include <vector>
 
 #include "arrow/array.h"
 #include "arrow/array/diff.h"
+#include "arrow/array/statistics.h"
 #include "arrow/buffer.h"
 #include "arrow/scalar.h"
 #include "arrow/sparse_tensor.h"
@@ -46,9 +49,9 @@
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/float16.h"
 #include "arrow/util/key_value_metadata.h"
-#include "arrow/util/logging.h"
+#include "arrow/util/logging_internal.h"
 #include "arrow/util/macros.h"
-#include "arrow/util/memory.h"
+#include "arrow/util/memory_internal.h"
 #include "arrow/util/ree_util.h"
 #include "arrow/visit_scalar_inline.h"
 #include "arrow/visit_type_inline.h"
@@ -1152,9 +1155,8 @@ bool ScalarEquals(const Scalar& left, const Scalar& right, const EqualOptions& o
 bool ArrayRangeEquals(const Array& left, const Array& right, int64_t left_start_idx,
                       int64_t left_end_idx, int64_t right_start_idx,
                       const EqualOptions& options) {
-  const bool floating_approximate = false;
   return ArrayRangeEquals(left, right, left_start_idx, left_end_idx, right_start_idx,
-                          options, floating_approximate);
+                          options, options.use_atol());
 }
 
 bool ArrayRangeApproxEquals(const Array& left, const Array& right, int64_t left_start_idx,
@@ -1166,8 +1168,7 @@ bool ArrayRangeApproxEquals(const Array& left, const Array& right, int64_t left_
 }
 
 bool ArrayEquals(const Array& left, const Array& right, const EqualOptions& opts) {
-  const bool floating_approximate = false;
-  return ArrayEquals(left, right, opts, floating_approximate);
+  return ArrayEquals(left, right, opts, opts.use_atol());
 }
 
 bool ArrayApproxEquals(const Array& left, const Array& right, const EqualOptions& opts) {
@@ -1176,8 +1177,7 @@ bool ArrayApproxEquals(const Array& left, const Array& right, const EqualOptions
 }
 
 bool ScalarEquals(const Scalar& left, const Scalar& right, const EqualOptions& options) {
-  const bool floating_approximate = false;
-  return ScalarEquals(left, right, options, floating_approximate);
+  return ScalarEquals(left, right, options, options.use_atol());
 }
 
 bool ScalarApproxEquals(const Scalar& left, const Scalar& right,
@@ -1523,4 +1523,55 @@ bool TypeEquals(const DataType& left, const DataType& right, bool check_metadata
   }
 }
 
+namespace {
+
+bool DoubleEquals(const double& left, const double& right, const EqualOptions& options) {
+  bool result;
+  auto visitor = [&](auto&& compare_func) { result = compare_func(left, right); };
+  VisitFloatingEquality<double>(options, options.use_atol(), std::move(visitor));
+  return result;
+}
+
+bool ArrayStatisticsValueTypeEquals(
+    const std::optional<ArrayStatistics::ValueType>& left,
+    const std::optional<ArrayStatistics::ValueType>& right, const EqualOptions& options) {
+  if (!left.has_value() || !right.has_value()) {
+    return left.has_value() == right.has_value();
+  } else if (left->index() != right->index()) {
+    return false;
+  } else {
+    auto EqualsVisitor = [&](const auto& v1, const auto& v2) {
+      using type_1 = std::decay_t<decltype(v1)>;
+      using type_2 = std::decay_t<decltype(v2)>;
+      if constexpr (std::conjunction_v<std::is_same<type_1, double>,
+                                       std::is_same<type_2, double>>) {
+        return DoubleEquals(v1, v2, options);
+      } else if constexpr (std::is_same_v<type_1, type_2>) {
+        return v1 == v2;
+      }
+      // It is unreachable
+      DCHECK(false);
+      return false;
+    };
+    return std::visit(EqualsVisitor, left.value(), right.value());
+  }
+}
+
+bool ArrayStatisticsEqualsImpl(const ArrayStatistics& left, const ArrayStatistics& right,
+                               const EqualOptions& equal_options) {
+  return left.null_count == right.null_count &&
+         left.distinct_count == right.distinct_count &&
+         left.is_min_exact == right.is_min_exact &&
+         left.is_max_exact == right.is_max_exact &&
+         ArrayStatisticsValueTypeEquals(left.min, right.min, equal_options) &&
+         ArrayStatisticsValueTypeEquals(left.max, right.max, equal_options);
+}
+
+}  // namespace
+
+bool ArrayStatisticsEquals(const ArrayStatistics& left, const ArrayStatistics& right,
+                           const EqualOptions& options) {
+  return ArrayStatisticsEqualsImpl(left, right, options);
+}
+
 }  // namespace arrow
