@@ -19,6 +19,7 @@
 #include <chrono>
 #include <condition_variable>
 #include <mutex>
+#include <optional>
 #include <random>
 #include <thread>
 #include <unordered_set>
@@ -31,7 +32,7 @@
 #include "arrow/type_fwd.h"
 #include "arrow/util/async_generator.h"
 #include "arrow/util/async_util.h"
-#include "arrow/util/optional.h"
+#include "arrow/util/config.h"
 #include "arrow/util/test_common.h"
 #include "arrow/util/vector.h"
 
@@ -333,7 +334,7 @@ TEST(TestAsyncUtil, MapAsync) {
   std::vector<TestInt> input = {1, 2, 3};
   auto generator = util::AsyncVectorIt(input);
   std::function<Future<TestStr>(const TestInt&)> mapper = [](const TestInt& in) {
-    return SleepAsync(1e-3).Then([in]() { return TestStr(std::to_string(in.value)); });
+    return SleepABitAsync().Then([in]() { return TestStr(std::to_string(in.value)); });
   };
   auto mapped = MakeMappedGenerator(std::move(generator), mapper);
   std::vector<TestStr> expected{"1", "2", "3"};
@@ -398,6 +399,10 @@ TEST(TestAsyncUtil, MapParallelStress) {
 }
 
 TEST(TestAsyncUtil, MapQueuingFailStress) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+
   constexpr int NTASKS = 10;
   constexpr int NITEMS = 10;
   for (bool slow : {true, false}) {
@@ -705,6 +710,7 @@ TEST_P(MergedGeneratorTestFixture, MergedLimitedSubscriptions) {
   AssertGeneratorExhausted(merged);
 }
 
+#ifndef ARROW_VALGRIND
 TEST_P(MergedGeneratorTestFixture, MergedStress) {
   constexpr int NGENERATORS = 10;
   constexpr int NITEMS = 10;
@@ -717,7 +723,7 @@ TEST_P(MergedGeneratorTestFixture, MergedStress) {
       sources.push_back(source);
     }
     AsyncGenerator<AsyncGenerator<TestInt>> source_gen = util::AsyncVectorIt(sources);
-    auto outer_gaurd = ExpectNotAccessedReentrantly(&source_gen);
+    auto outer_guard = ExpectNotAccessedReentrantly(&source_gen);
 
     auto merged = MakeMergedGenerator(source_gen, 4);
     ASSERT_FINISHES_OK_AND_ASSIGN(auto items, CollectAsyncGenerator(merged));
@@ -739,6 +745,7 @@ TEST_P(MergedGeneratorTestFixture, MergedParallelStress) {
     ASSERT_EQ(NITEMS * NGENERATORS, items.size());
   }
 }
+#endif
 
 TEST_P(MergedGeneratorTestFixture, MergedRecursion) {
   // Regression test for an edge case in MergedGenerator. Ensure if
@@ -762,6 +769,25 @@ TEST_P(MergedGeneratorTestFixture, MergedRecursion) {
   }
 }
 
+TEST_P(MergedGeneratorTestFixture, DeepOuterGeneratorStackOverflow) {
+  // Simulate a very deep and very quick outer generator that yields simple
+  // inner generators.  Everything completes synchronously.  This is to
+  // try and provoke a stack overflow the simulates ARROW-16692
+  constexpr int kNumItems = 10000;
+  constexpr int kMaxSubscriptions = 8;
+  std::vector<AsyncGenerator<TestInt>> inner_generators;
+  for (int i = 0; i < kNumItems; i++) {
+    inner_generators.push_back(MakeVectorGenerator<TestInt>({}));
+  }
+  AsyncGenerator<AsyncGenerator<TestInt>> outer_generator =
+      MakeVectorGenerator(inner_generators);
+  AsyncGenerator<TestInt> merged =
+      MakeMergedGenerator(outer_generator, kMaxSubscriptions);
+  ASSERT_FINISHES_OK_AND_ASSIGN(std::vector<TestInt> collected,
+                                CollectAsyncGenerator(std::move(merged)));
+  ASSERT_TRUE(collected.empty());
+}
+
 INSTANTIATE_TEST_SUITE_P(MergedGeneratorTests, MergedGeneratorTestFixture,
                          ::testing::Values(false, true));
 
@@ -973,6 +999,9 @@ TEST(TestAsyncUtil, GeneratorIterator) {
 }
 
 TEST(TestAsyncUtil, MakeTransferredGenerator) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
   std::mutex mutex;
   std::condition_variable cv;
   std::atomic<bool> finished(false);
@@ -1070,7 +1099,7 @@ TEST_P(BackgroundGeneratorTestFixture, BadResult) {
   ASSERT_FINISHES_OK_AND_EQ(TestInt(1), generator());
   // Next three results may or may not be valid.
   // The typical case is the call for TestInt(2) restarts a full queue and then maybe
-  // TestInt(3) and TestInt(4) arrive quickly enough to not get pre-empted or maybe
+  // TestInt(3) and TestInt(4) arrive quickly enough to not get preempted or maybe
   // they don't.
   //
   // A more bizarre, but possible, case is the checking thread falls behind the producer
@@ -1457,6 +1486,10 @@ TEST(TestAsyncUtil, ReadaheadMove) {
 }
 
 TEST(TestAsyncUtil, ReadaheadFailed) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+
   ASSERT_OK_AND_ASSIGN(auto thread_pool, internal::ThreadPool::Make(20));
   std::atomic<int32_t> counter(0);
   auto gating_task = GatingTask::Make();
@@ -1491,6 +1524,9 @@ TEST(TestAsyncUtil, ReadaheadFailed) {
 }
 
 TEST(TestAsyncUtil, ReadaheadFailedWaitForInFlight) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
   ASSERT_OK_AND_ASSIGN(auto thread_pool, internal::ThreadPool::Make(20));
   // If a failure causes an early end then we should not emit that failure
   // until all in-flight futures have completed.  This is to prevent tasks from
@@ -1827,7 +1863,7 @@ TEST(PushGenerator, CloseEarly) {
 }
 
 TEST(PushGenerator, DanglingProducer) {
-  util::optional<PushGenerator<TestInt>> gen;
+  std::optional<PushGenerator<TestInt>> gen;
   gen.emplace();
   auto producer = gen->producer();
 
@@ -1840,6 +1876,10 @@ TEST(PushGenerator, DanglingProducer) {
 }
 
 TEST(PushGenerator, Stress) {
+#ifndef ARROW_ENABLE_THREADING
+  GTEST_SKIP() << "Test requires threading support";
+#endif
+
   const int NTHREADS = 20;
   const int NVALUES = 2000;
   const int NFUTURES = NVALUES + 100;
