@@ -15,7 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include <cmath>
+#include <limits>
 #include <string>
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -23,6 +26,7 @@
 
 #include "arrow/status.h"
 #include "arrow/testing/gtest_util.h"
+#include "arrow/util/regex.h"
 #include "arrow/util/string.h"
 
 namespace arrow {
@@ -132,7 +136,7 @@ TEST(SplitString, OuterLeftAndOuterRightDelimiter) {
   EXPECT_EQ(parts[4], "");
 }
 
-TEST(SplitString, OnlyDemiliter) {
+TEST(SplitString, OnlyDelimiter) {
   std::string input = ":";
   auto parts = SplitString(input, ':');
   ASSERT_EQ(parts.size(), 2);
@@ -194,5 +198,87 @@ TEST(EndsWith, Basics) {
   ASSERT_FALSE(EndsWith(abcdef, abc));
 }
 
+TEST(RegexMatch, Basics) {
+  std::regex regex("a+(b*)(c+)d+");
+  std::string_view b, c;
+
+  ASSERT_FALSE(RegexMatch(regex, "", {&b, &c}));
+  ASSERT_FALSE(RegexMatch(regex, "ad", {&b, &c}));
+  ASSERT_FALSE(RegexMatch(regex, "bc", {&b, &c}));
+
+  auto check_match = [&](std::string_view target, std::string_view expected_b,
+                         std::string_view expected_c) {
+    b = c = "!!!";  // dummy init value
+    ASSERT_TRUE(RegexMatch(regex, target, {&b, &c}));
+    ASSERT_EQ(b, expected_b);
+    ASSERT_EQ(c, expected_c);
+  };
+
+  check_match("abcd", "b", "c");
+  check_match("acd", "", "c");
+  check_match("abbcccd", "bb", "ccc");
+}
+
+TEST(ToChars, Integers) {
+  ASSERT_EQ(ToChars(static_cast<char>(0)), "0");
+  ASSERT_EQ(ToChars(static_cast<unsigned char>(0)), "0");
+  ASSERT_EQ(ToChars(static_cast<int8_t>(0)), "0");
+  ASSERT_EQ(ToChars(static_cast<uint64_t>(0)), "0");
+  ASSERT_EQ(ToChars(1234), "1234");
+  ASSERT_EQ(ToChars(-5678), "-5678");
+
+  if constexpr (have_to_chars<int>) {
+    ASSERT_EQ(ToChars(1234, /*base=*/2), "10011010010");
+  }
+
+  // Beyond pre-allocated result size
+  ASSERT_EQ(ToChars(9223372036854775807LL), "9223372036854775807");
+  ASSERT_EQ(ToChars(-9223372036854775807LL - 1), "-9223372036854775808");
+  ASSERT_EQ(ToChars(18446744073709551615ULL), "18446744073709551615");
+
+  if constexpr (have_to_chars<unsigned long long>) {  // NOLINT: runtime/int
+    // Will overflow any small string optimization
+    ASSERT_EQ(ToChars(18446744073709551615ULL, /*base=*/2),
+              "1111111111111111111111111111111111111111111111111111111111111111");
+  }
+}
+
+TEST(ToChars, FloatingPoint) {
+  if constexpr (have_to_chars<double>) {
+    ASSERT_EQ(ToChars(0.0f), "0");
+    ASSERT_EQ(ToChars(0.0), "0");
+    ASSERT_EQ(ToChars(-0.0), "-0");
+    ASSERT_EQ(ToChars(0.25), "0.25");
+    ASSERT_EQ(ToChars(-0.25f), "-0.25");
+
+    ASSERT_EQ(ToChars(0.1111111111111111), "0.1111111111111111");
+
+    // XXX Can't test std::chars_format as it's not defined by all standard libraries
+    // and even "if constexpr" wouldn't prevent the failing lookup.
+  } else {
+    // If std::to_chars isn't implemented for floating-point types, we fall back
+    // to std::to_string which may make ad hoc formatting choices, so we cannot
+    // really test much about the result.
+    auto result = ToChars(0.0f);
+    ASSERT_TRUE(StartsWith(result, "0")) << result;
+    result = ToChars(0.25);
+    ASSERT_TRUE(StartsWith(result, "0.25")) << result;
+  }
+}
+
+#if !defined(_WIN32) || defined(NDEBUG)
+
+TEST(ToChars, LocaleIndependent) {
+  if constexpr (have_to_chars<double>) {
+    // French locale uses the comma as decimal point
+    LocaleGuard locale_guard("fr_FR.UTF-8");
+
+    ASSERT_EQ(ToChars(0.25), "0.25");
+    ASSERT_EQ(ToChars(-0.25f), "-0.25");
+  }
+}
+
+#endif  // _WIN32
+
 }  // namespace internal
 }  // namespace arrow
