@@ -34,6 +34,8 @@
 #include <rapidjson/stringbuffer.h>
 
 #include "arrow/array.h"
+#include "arrow/array/array_binary.h"
+#include "arrow/array/builder_binary.h"
 #include "arrow/buffer.h"
 #include "arrow/io/file.h"
 #include "arrow/testing/future_util.h"
@@ -45,13 +47,17 @@
 
 #include "parquet/column_reader.h"
 #include "parquet/column_scanner.h"
+#include "parquet/column_writer.h"
 #include "parquet/file_reader.h"
 #include "parquet/file_writer.h"
+#include "parquet/geospatial/statistics.h"
 #include "parquet/metadata.h"
 #include "parquet/page_index.h"
 #include "parquet/platform.h"
 #include "parquet/printer.h"
+#include "parquet/statistics.h"
 #include "parquet/test_util.h"
+#include "parquet/types.h"
 
 namespace rj = arrow::rapidjson;
 
@@ -1857,4 +1863,178 @@ TEST(PageIndexReaderTest, ReadFileWithoutPageIndex) {
   ASSERT_EQ(nullptr, row_group_index_reader);
 }
 
+class TestGeometryLogicalType : public ::testing::Test {
+ public:
+  const int kNumRows = 1000;
+
+  void WriteTestData(std::shared_ptr<const LogicalType> type, bool write_arrow) {
+    // Make schema
+    schema::NodeVector fields;
+    fields.push_back(
+        PrimitiveNode::Make("g", Repetition::REQUIRED, type, Type::BYTE_ARRAY));
+    auto schema = std::static_pointer_cast<GroupNode>(
+        GroupNode::Make("schema", Repetition::REQUIRED, fields));
+
+    // Write small batches and small data pages
+    auto writer_props_builder = WriterProperties::Builder();
+    writer_props_builder.write_batch_size(64);
+
+    std::shared_ptr<WriterProperties> writer_props = writer_props_builder.build();
+
+    ASSERT_OK_AND_ASSIGN(auto out_file, ::arrow::io::BufferOutputStream::Create());
+    std::shared_ptr<ParquetFileWriter> file_writer =
+        ParquetFileWriter::Open(out_file, schema, writer_props);
+    RowGroupWriter* rg_writer = file_writer->AppendRowGroup();
+
+    // write WKB points to columns
+    auto* writer =
+        ::arrow::internal::checked_cast<ByteArrayWriter*>(rg_writer->NextColumn());
+    if (!write_arrow) {
+      WriteTestDataUsingWriteBatch(writer);
+    } else {
+      WriteTestDataUsingWriteArrow(writer);
+    }
+
+    rg_writer->Close();
+    file_writer->Close();
+
+    ASSERT_OK_AND_ASSIGN(file_buf, out_file->Finish());
+  }
+
+  void WriteTestDataUsingWriteBatch(ByteArrayWriter* writer) {
+    std::vector<uint8_t> buffer(test::kWkbPointXYSize * kNumRows);
+    uint8_t* ptr = buffer.data();
+    std::vector<ByteArray> values(kNumRows);
+    for (int k = 0; k < kNumRows; k++) {
+      std::string item = test::MakeWKBPoint(
+          {static_cast<double>(k), static_cast<double>(k + 1)}, false, false);
+      std::memcpy(ptr, item.data(), item.size());
+      values[k].len = test::kWkbPointXYSize;
+      values[k].ptr = ptr;
+      ptr += test::kWkbPointXYSize;
+    }
+    writer->WriteBatch(kNumRows, nullptr, nullptr, values.data());
+  }
+
+  void WriteTestDataUsingWriteArrow(ByteArrayWriter* writer) {
+    ::arrow::BinaryBuilder builder;
+    for (int k = 0; k < kNumRows; k++) {
+      std::string item = test::MakeWKBPoint(
+          {static_cast<double>(k), static_cast<double>(k + 1)}, false, false);
+
+      ASSERT_OK(builder.Append(item));
+    }
+    std::shared_ptr<::arrow::BinaryArray> array;
+    ASSERT_OK(builder.Finish(&array));
+
+    std::shared_ptr<ArrowWriterProperties> properties =
+        ArrowWriterProperties::Builder().build();
+    MemoryPool* pool = ::arrow::default_memory_pool();
+    auto ctx = std::make_unique<ArrowWriteContext>(pool, properties.get());
+    ASSERT_OK(writer->WriteArrow(nullptr, nullptr, kNumRows, *array, ctx.get(),
+                                 /*leaf_field_nullable=*/true));
+  }
+
+  void TestWriteAndRead(std::shared_ptr<const LogicalType> type, bool write_arrow) {
+    ASSERT_NO_FATAL_FAILURE(WriteTestData(type, write_arrow));
+
+    auto in_file = std::make_shared<::arrow::io::BufferReader>(file_buf);
+
+    ReaderProperties reader_props;
+    reader_props.enable_buffered_stream();
+    reader_props.set_buffer_size(64);
+    auto file_reader = ParquetFileReader::Open(in_file, reader_props);
+
+    // Check that the geometry statistics are correctly written and read
+    auto metadata = file_reader->metadata();
+    ASSERT_TRUE(type->Equals(*metadata->schema()->Column(0)->logical_type()));
+
+    auto page_index_reader = file_reader->GetPageIndexReader();
+    int num_row_groups = metadata->num_row_groups();
+    int64_t start_index = 0;
+    for (int i = 0; i < num_row_groups; i++) {
+      auto row_group_metadata = metadata->RowGroup(i);
+      auto column_chunk_metadata = row_group_metadata->ColumnChunk(0);
+      auto geo_stats = column_chunk_metadata->geo_statistics();
+      ASSERT_NO_FATAL_FAILURE(CheckGeoStatistics(type, geo_stats, start_index,
+                                                 row_group_metadata->num_rows()));
+      start_index += row_group_metadata->num_rows();
+    }
+
+    // Check the geometry values
+    int64_t total_values_read = 0;
+    for (int i = 0; i < num_row_groups; i++) {
+      auto row_group = file_reader->RowGroup(i);
+      std::shared_ptr<ByteArrayReader> reader =
+          std::static_pointer_cast<ByteArrayReader>(row_group->Column(0));
+      while (reader->HasNext()) {
+        std::vector<ByteArray> out(kNumRows);
+        int64_t values_read = 0;
+        int64_t levels_read =
+            reader->ReadBatch(kNumRows, nullptr, nullptr, out.data(), &values_read);
+        ASSERT_GE(levels_read, 1);
+        ASSERT_GE(values_read, 1);
+
+        // Check the batch
+        for (int64_t i = 0; i < values_read; i++) {
+          const ByteArray& value = out[i];
+          auto xy = test::GetWKBPointCoordinateXY(value);
+          EXPECT_TRUE(xy.has_value());
+          auto expected_x = static_cast<double>(i + total_values_read);
+          auto expected_y = static_cast<double>(i + 1 + total_values_read);
+          EXPECT_EQ(*xy, (std::pair<double, double>(expected_x, expected_y)));
+        }
+
+        total_values_read += values_read;
+      }
+    }
+    EXPECT_EQ(kNumRows, total_values_read);
+  }
+
+  void CheckGeoStatistics(std::shared_ptr<const LogicalType> type,
+                          std::shared_ptr<geospatial::GeoStatistics> geom_stats,
+                          int64_t start_index, int64_t num_rows) {
+    // We don't yet generate statistics for Geography
+    if (type->is_geography()) {
+      ASSERT_EQ(geom_stats, nullptr);
+      return;
+    }
+
+    ASSERT_NE(geom_stats, nullptr);
+    // We wrote exactly one geometry type (POINT, which has code 1)
+    EXPECT_THAT(*geom_stats->geometry_types(), ::testing::ElementsAre(1));
+
+    double expected_xmin = static_cast<double>(start_index);
+    double expected_xmax = expected_xmin + num_rows - 1;
+    double expected_ymin = expected_xmin + 1;
+    double expected_ymax = expected_xmax + 1;
+
+    EXPECT_EQ(geom_stats->lower_bound()[0], expected_xmin);
+    EXPECT_EQ(geom_stats->upper_bound()[0], expected_xmax);
+    EXPECT_EQ(geom_stats->lower_bound()[1], expected_ymin);
+    EXPECT_EQ(geom_stats->upper_bound()[1], expected_ymax);
+    EXPECT_THAT(geom_stats->dimension_valid(),
+                ::testing::ElementsAre(true, true, false, false));
+  }
+
+ protected:
+  std::shared_ptr<Buffer> file_buf;
+};
+
+TEST_F(TestGeometryLogicalType, TestWriteGeometry) {
+  TestWriteAndRead(GeometryLogicalType::Make("srid:1234"), /*write_arrow=*/false);
+}
+
+TEST_F(TestGeometryLogicalType, TestWriteArrowAndReadGeometry) {
+  TestWriteAndRead(GeometryLogicalType::Make("srid:1234"), /*write_arrow=*/true);
+}
+
+TEST_F(TestGeometryLogicalType, TestWriteGeography) {
+  TestWriteAndRead(GeographyLogicalType::Make("srid:1234"), /*write_arrow=*/false);
+}
+
+TEST_F(TestGeometryLogicalType, TestWriteGeographyArrow) {
+  TestWriteAndRead(GeographyLogicalType::Make("srid:1234"), /*write_arrow=*/true);
+}
+
 }  // namespace parquet
